// Copyright(C) 2005 NSP. All rights reserved.
using System;
using System.Drawing;
using System.Windows.Forms;
using System.Drawing.Drawing2D;

class GraphPanel : PictureBox {
	Bitmap image;
	GraphDrawer graph;
	public GraphPanel(){
		image = new Bitmap(640, 480);
		this.Image = image;
		graph = new GraphDrawer();
		graph.Graphics = Graphics.FromImage(image);
		graph.Size = image.Size;
		graph.Xrange = new GraphRange(0, 23*60*60+10*60);
		graph.Yrange = new GraphRange(0, 800);
		string [] xtics = {"00:00", "03:00", "06:00", "09:00", "12:00", "15:00", "18:00", "21:00"};
		int [] xpos = new int[xtics.Length];
		string [] ytics = {"0","400", "800"};
		int [] ypos = {0, 400, 800};
		int x = 0;
		int dx = 3*60*60;
		for(int i = 0; i < xpos.Length; i++){
			xpos[i] = x;
			x+=dx;
		}
		graph.setXtics(xtics, xpos);
		graph.setYtics(ytics, ypos);
		graph.XSubTicsStep = 3600;
		graph.YSubTicsStep = 100;
		graph.PointSize = 1.3;
		graph.BoxWidth = 0.3;
		
		Point [] ps = new Point[24];
		for(int i = 0; i < ps.Length; i++){
			ps[i] = new Point(3600*i, i*i*2);
		}
		Point [] ps2 = new Point[24];
		for(int i = 0; i < ps.Length; i++){
			ps2[i] = new Point(3600*i, i*i);
		}
		
		Point [] ps3 = new Point[24];
		for(int i = 0; i < ps.Length; i++){
			ps3[i] = new Point(3600*i, i*i/2);
		}
		
		GraphItem gi = new GraphItem();
		gi.Points = ps;
		gi.Color = Color.Red;
		gi.Style = GraphItemStyle.LinesPoints;
		gi.Title = "Test1";
		GraphItem gi2 = new GraphItem();
		gi2.Points = ps2;
		gi2.Color = Color.Blue;
		gi2.Style = GraphItemStyle.Box;
		gi2.Title = "TestTest2";
		GraphItem gi3 = new GraphItem();
		gi3.Points = ps3;
		gi3.Color = Color.Green;
		gi3.Style = GraphItemStyle.Lines;
		gi3.Title = "TestTest3";
		graph.Plot(new GraphItem [] {gi, gi2, gi3});
	}
}

// Ot̎̃W(min, max)
struct GraphRange {
	int min;         
	int max;
	public int Min {
		set{this.min = value;}
		get{return this.min;}
	}
	public int Max {
		set{this.max = value;}
		get{return this.max;}
	}
	public GraphRange(int min, int max){
		this.min = min;
		this.max = max;
	}
}

/* Graphics ɃOt`
 * f[^ GraphItem ̔zœn
 * Xtics ͈ʒuƃx̔zn
 * Ytics lD
 * ̂Ƃ낱w肵ĂȂƂîj
 * subtics ̓CNgw肷(min炻̕ŕ`)
 * range w肵Ȃƃ_îI[gHj
 * Ot̉摜ƂẴTCY size w肷ׂ
 */ 
class GraphDrawer {
	string [] xticsLabel;  // tics֌W
	string [] yticsLabel;
	int [] xticsPos;
	int [] yticsPos;
	int xsubticsStep;      // subtics
	int ysubticsStep;
	double boxWidth;       // box ł̕
	double pointSize;      // points ̂Ƃ
	Graphics graphics;
	Size size;             // OtŜ̉摜ƂẴTCY
	GraphRange xrange;     // W
	GraphRange yrange;

	Color [] defaultColor = {Color.Red, Color.Blue, Color.Green};

	
	public GraphDrawer(){
		size = new Size(640, 480);
		xrange = new GraphRange(0, 64);
		yrange = new GraphRange(0, 48);
		boxWidth = 1.0;
		pointSize = 1.0;
		xsubticsStep = 0;
		ysubticsStep = 0;
	}
	public double BoxWidth {
		set{this.boxWidth = value;}
		get{return this.boxWidth;}
	}
	public double PointSize {
		set{this.pointSize = value;}
		get{return this.pointSize;}
	}
	public int XSubTicsStep {
		set{this.xsubticsStep = value;}
		get{return this.xsubticsStep;}
	}
	public int YSubTicsStep {
		set{this.ysubticsStep = value;}
		get{return this.ysubticsStep;}
	}
	public Graphics Graphics {
		set{this.graphics = value;}
		get{return this.graphics;}
	}
	public Size Size {
		set{this.size = value;}
		get{return this.size;}
	}
	public GraphRange Xrange {
		set{this.xrange = value;}
		get{return this.xrange;}
	}
	public GraphRange Yrange {
		set{this.yrange = value;}
		get{return this.yrange;}
	}
	public void setXtics(string [] label, int [] pos){
		xticsLabel = label;
		xticsPos = pos;
	}
	public void setYtics(string [] label, int [] pos){
		yticsLabel = label;
		yticsPos = pos;
	}
	public void Plot(GraphItem [] items){
		if(graphics==null) return;
		GraphParams gp = new GraphParams(this);
		// Ot`̈̎l
		int left = gp.left;
		int right = gp.right;
		int top = gp.top;
		int bottom = gp.bottom;

		// NA
		graphics.FillRectangle(gp.bgBrush,0,0,size.Width,size.Height);

		

		// ϊs
		Rectangle rectS = new Rectangle(xrange.Min, yrange.Min, xrange.Max, yrange.Max);
		Point [] rectD = new Point[3];
		rectD[0] = new Point(left, bottom);
		rectD[1] = new Point(right, bottom);
		rectD[2] = new Point(left, top);
		Matrix trans = new Matrix(rectS, rectD);
		
		// ϊĕ`悷̂p
		TransformGraphics g = new TransformGraphics(graphics, trans);

		// ϊ̗̈̏
		int width = gp.width;
		int height = gp.height;
		float ratioX = gp.ratioX;
		float ratioY = gp.ratioY;

		Brush axisBrush = gp.axisBrush;
		Brush stringBrush = gp.stringBrush;
		Pen axisPen = gp.axisPen;
		Font font = gp.font;
		// ̕`ʒu̒p
		StringFormat sf = gp.sf;

		// Og
		g.DrawRectangle(axisPen,xrange.Min, yrange.Min, width, height);

		// xtics
		int ticsH = (int)(gp.ticsHeight * height);
		float strY =(float)(yrange.Min);
		sf.Alignment = StringAlignment.Center;    // ͐^񒆂 
		for(int i = 0; i < xticsPos.Length; i++){
			String label = (xticsLabel[i]==null) ? "" : xticsLabel[i];
			g.DrawLine(axisPen, xticsPos[i], yrange.Min, xticsPos[i], yrange.Min + ticsH);
			g.DrawLine(axisPen, xticsPos[i], yrange.Max, xticsPos[i], yrange.Max-ticsH);
			PointF pf = new PointF(xticsPos[i], strY);
			g.DrawString(label, font, stringBrush, pf, sf);
		}
		// ytics
		int ticsW = (int)(gp.ticsHeight * width);
		float strX = (float)(xrange.Min);
		float zeroOff = (float)(font.Height*ratioY/2);  // ͂߂͏ɂ炷
		sf.Alignment = StringAlignment.Far;         // ɕ񏑂
		sf.LineAlignment = StringAlignment.Center;  // c͐^񒆂
		for(int i = 0; i < yticsPos.Length; i++){
			String label = (yticsLabel[i]==null) ? "" : yticsLabel[i];
			g.DrawLine(axisPen, xrange.Min, yticsPos[i], xrange.Min + ticsW, yticsPos[i]);
			g.DrawLine(axisPen, xrange.Max, yticsPos[i], xrange.Max-ticsW, yticsPos[i]);
			PointF pf = new PointF(strX, yticsPos[i]+ zeroOff);
			zeroOff = 0;
			g.DrawString(label, font, stringBrush, pf, sf);
		}
		// xsubtics
		ticsH = (int)(gp.subticsHeight * height);
		if(xsubticsStep>0){
			for(int x = xrange.Min; x < xrange.Max; x+=xsubticsStep){
				g.DrawLine(axisPen, x, yrange.Min, x, yrange.Min + ticsH);
				g.DrawLine(axisPen, x, yrange.Max, x, yrange.Max-ticsH);
				
			}
		}
		// ysubtics
		ticsW = (int)(gp.subticsHeight * width);
		if(xsubticsStep>0){
			for(int y = yrange.Min; y < yrange.Max; y+=ysubticsStep){
				g.DrawLine(axisPen, xrange.Min, y, xrange.Min + ticsW, y);
				g.DrawLine(axisPen, xrange.Max, y, xrange.Max - ticsW, y);
				
			}
		}

		// data
		Region r = graphics.Clip;

		// Ot͂ݏoȂ悤ɃNbv̈
		Region r2 = new Region(new Rectangle(left, top, right-left+1, bottom-top+1));
		// Points ͂Ƃ͂ݏoĂ
		int d = 5;
		Region r3 = new Region(new Rectangle(left-d, top-d, right-left+1+d+d, bottom-top+1+d+d));
		for(int i = 0; i < items.Length; i++){
			if(items[i].Points==null) continue;
			if(items[i].Points.Length==0) continue;
			switch(items[i].Style){              // X^CŊU
			case GraphItemStyle.Lines:
				graphics.Clip = r2;
				drawItemLines(g, items[i], i, gp);
				break;
			case GraphItemStyle.Points:
				graphics.Clip = r3;
				drawItemPoints(g, items[i], i, gp);
				break;
			case GraphItemStyle.Box:
				graphics.Clip = r2;
				drawItemBox(g, items[i], i, gp);
				break;
			case GraphItemStyle.LinesPoints:
				graphics.Clip = r2;
				drawItemLines(g, items[i], i, gp);
				graphics.Clip = r3;
				drawItemPoints(g, items[i], i, gp);
				break;
			}
			graphics.Clip = r;
		}
		//g.Transform = transOrg;
	}
	// `̂߂̃p[^ێ
	class GraphParams{
		// Ot`̗̈]Ctics ̍̊
		public double marginTop = 0.05;      // c̒ɑ΂
		public double marginBottom = 0.12;
		public double marginLeft = 0.12;     // ̒ɑ΂
		public double marginRight = 0.05;
		public double ticsHeight = 0.012;     // c̒ɑ΂
		public double subticsHeight = 0.004;
		public double keyBaseXRatio = 0.08;
		public double keyBaseYRatio = 0.05;
		public double keyStepRatio = -0.05;
		public double keyLengthRatio = 0.06;
		public double keyHeightRatio = 0.015;
		public int left, right, top, bottom; // `̈
		public int width, height;            
		public int keyBaseX, keyBaseY;
		public int keyStep;
		public int keyLength;
		public int keyHeight;
		
		public float ratioX, ratioY;         // ʂƂ̔
		public Brush bgBrush;
		GraphDrawer gd;
		public Brush axisBrush;
		public Brush stringBrush;
		public Pen axisPen;
		public Font font;
		public StringFormat sf, key_sf;
		public GraphParams(GraphDrawer gd){
			this.gd = gd;
			left = (int)(gd.size.Width * marginLeft);
			right = (int)(gd.size.Width * (1-marginRight));
			top = (int)(gd.size.Height * marginTop);
			bottom = (int)(gd.size.Height * (1-marginBottom));
			bgBrush = new SolidBrush(Color.White);
			width = gd.xrange.Max - gd.xrange.Min;
			height = gd.yrange.Max - gd.yrange.Min;
			ratioX = width / (float) gd.size.Width;
			ratioY = height / (float) gd.size.Height;
			keyBaseX = (int)(gd.size.Width * keyBaseXRatio*ratioX);
			keyBaseY = (int)(gd.size.Height * (1-keyBaseYRatio)*ratioY);
			keyStep = (int)(gd.size.Height*keyStepRatio*ratioY);
			keyLength = (int)(gd.size.Width*keyLengthRatio*ratioX);
			keyHeight = (int)(gd.size.Height*keyHeightRatio*ratioY);
			// uVƂtHgp
			axisBrush = new SolidBrush(Color.Black);
			stringBrush = new SolidBrush(Color.Black);
			axisPen = new Pen(axisBrush, 1f);
			font = new Font("Arial", 10);
			sf = new StringFormat(StringFormat.GenericDefault);
			key_sf = new StringFormat(StringFormat.GenericDefault);
			key_sf.LineAlignment = StringAlignment.Center;
			key_sf.Alignment = StringAlignment.Near;
		}
	}
	// Lines ̕`
	void drawItemLines(TransformGraphics g, GraphItem item, int num, GraphParams gp){
		Point [] ps = item.Points;
		Point p = ps[0];
		Pen pen = new Pen(new SolidBrush(item.Color), item.Width);
		for(int i = 1; i < ps.Length; i++){
			
			g.DrawLine(pen, p, ps[i]);
			p = ps[i];
		}
		if(item.Title!=null){
			PointF pf = new Point(gp.keyBaseX, gp.keyBaseY + num * gp.keyStep);
			g.DrawString(" " + item.Title, gp.font, gp.stringBrush, pf, gp.key_sf);
			PointF pf2 = new Point(gp.keyBaseX-gp.keyLength, gp.keyBaseY + num * gp.keyStep);
			g.DrawLine(pen, pf, pf2);
			
		}
	}
	// Points ̕`
	void drawItemPoints(TransformGraphics g, GraphItem item, int num, GraphParams gp){
		Point [] ps = item.Points;
		float r = (float)(3*pointSize); // a
		float rrX = r * gp.ratioX;             // XP[O̔a
		float rrY = r * gp.ratioY;
		float rrX2 = rrX * 2;           // XP[O̒a
		float rrY2 = rrY * 2;
		SolidBrush b = new SolidBrush(item.Color);
		//Pen pen = new Pen(b, item.Width*4);
		for(int i = 0; i < ps.Length; i++){
			//g.DrawEllipse(pen, ps[i].X-rrX, ps[i].Y - rrY,rrX2,rrY2);
			g.FillEllipse(b, ps[i].X-rrX, ps[i].Y - rrY,rrX2,rrY2);
		}
		if(item.Title!=null){
			PointF pf = new Point(gp.keyBaseX, gp.keyBaseY + num * gp.keyStep);
			g.DrawString(" " + item.Title, gp.font, gp.stringBrush, pf, gp.key_sf);
			g.FillEllipse(b, gp.keyBaseX-gp.keyLength/2-rrX, gp.keyBaseY - rrY,rrX2,rrY2);
			
		}
	}
	// Box ̕`
	void drawItemBox(TransformGraphics g, GraphItem item, int num, GraphParams gp){
		Point [] ps = item.Points;
		Pen pen = new Pen(new SolidBrush(item.Color), item.Width);
		PointF [] ps2 = new PointF[4];
		int dist = xrange.Max - xrange.Min;     // ŏ̕߂
		for(int i = 1; i < ps.Length; i++){
			int dd = ps[i].X - ps[i-1].X;
			if(dd < 0) dd = -dd;
			if(dd!=0 && dd < dist) dist = dd;
		}
		float d = (float)(dist * boxWidth / 2);  // ̕œꂷ
		for(int i = 0; i < ps.Length; i++){
			float left = ps[i].X - d;
			float right = ps[i].X + d;
			ps2[0] = new PointF(left, yrange.Min);
			ps2[1] = new PointF(left, ps[i].Y);
			ps2[2] = new PointF(right, ps[i].Y);
			ps2[3] = new PointF(right, yrange.Min);
			g.DrawLines(pen, ps2);
		}
		if(item.Title!=null){
			PointF pf = new Point(gp.keyBaseX, gp.keyBaseY + num * gp.keyStep);
			g.DrawString(" " + item.Title, gp.font, gp.stringBrush, pf, gp.key_sf);
			g.DrawRectangle(pen, (int)(pf.X - gp.keyLength), (int)(pf.Y - gp.keyHeight/2), gp.keyLength , gp.keyHeight);
			
		}
	}
	
	// uVƂ̈ςɍŴ݂ϊ
	// Graphics.Transform ƃuVɂȂ肷邩
	// 肽͈̂ȉ̂
	// Matrix transOrg = g.Transform;
	// trans.Multiply(transOrg, MatrixOrder.Append);
	// g.Transform = trans;
	class TransformGraphics {
		Graphics g;
		Matrix trans;
		public TransformGraphics(Graphics g, Matrix t){
			this.g = g;
			this.trans = t;
		}
		public void DrawLine(Pen p, int x1, int y1, int x2, int y2){
			Point [] ps = {new Point(x1, y1), new Point(x2, y2)};
			trans.TransformPoints(ps);
			g.DrawLine(p, ps[0], ps[1]);
		}
		public void DrawLine(Pen p, Point p1, Point p2){
			Point [] ps = {p1, p2};
			trans.TransformPoints(ps);
			g.DrawLine(p, ps[0], ps[1]);
		}
		public void DrawLine(Pen p, PointF p1, PointF p2){
			PointF [] ps = {p1, p2};
			trans.TransformPoints(ps);
			g.DrawLine(p, ps[0], ps[1]);
		}
		public void DrawRectangle(Pen p, int x, int y, int w, int h){
			Point [] ps = {new Point(x, y)};
			Point [] vs = {new Point(w, h)};
			transformRect(ps, vs);
			g.DrawRectangle(p, ps[0].X, ps[0].Y, vs[0].X, vs[0].Y);
		}
		public void FillRectangle(Brush b, int x, int y, int w, int h){
			Point [] ps = {new Point(x, y)};
			Point [] vs = {new Point(w, h)};
			transformRect(ps, vs);
			g.FillRectangle(b, ps[0].X, ps[0].Y, vs[0].X, vs[0].Y);
		}
		public void DrawString(string s, Font f, Brush b, PointF p){
			PointF [] ps = {p};
			trans.TransformPoints(ps);
			g.DrawString(s, f, b, ps[0]);
			
		}
		public void DrawString(string s, Font f, Brush b, PointF p, StringFormat sf){
			PointF [] ps = {p};
			trans.TransformPoints(ps);
			g.DrawString(s, f, b, ps[0],sf);
			
		}
		public void DrawEllipse(Pen pen, float x, float y, float w, float h){
			PointF [] ps = {new PointF(x, y)};
			PointF [] vs = {new PointF(w, h)};
			transformRect(ps, vs);
			g.DrawEllipse(pen, ps[0].X, ps[0].Y, vs[0].X, vs[0].Y);
		}
		public void FillEllipse(Brush b, float x, float y, float w, float h){
			PointF [] ps = {new PointF(x, y)};
			PointF [] vs = {new PointF(w, h)};
			transformRect(ps, vs);
			g.FillEllipse(b, ps[0].X, ps[0].Y, vs[0].X, vs[0].Y);
		}
		public void DrawLines(Pen pen, PointF [] ps){
			trans.TransformPoints(ps);
			g.DrawLines(pen, ps);
		}
		// WƕϊCƂɂȂȂ悤ɒ
		void transformRect(PointF [] ps, PointF [] vs){
			trans.TransformPoints(ps);
			trans.TransformVectors(vs);
			if(vs[0].Y < 0){
				ps[0].Y += vs[0].Y;
				vs[0].Y = -vs[0].Y;
			}
			if(vs[0].X < 0){
				ps[0].X += vs[0].X;
				vs[0].X = -vs[0].X;
			}
		}
		void transformRect(Point [] ps, Point [] vs){
			trans.TransformPoints(ps);
			trans.TransformVectors(vs);
			if(vs[0].Y < 0){
				ps[0].Y += vs[0].Y;
				vs[0].Y = -vs[0].Y;
			}
			if(vs[0].X < 0){
				ps[0].X += vs[0].X;
				vs[0].X = -vs[0].X;
			}
		}
	}
	
}

// Otɕ`悷ЂƂ̃f[^W
public class GraphItem {
	Color color;            // F
	GraphItemStyle style;   // X^C
	Point [] points;        // f[^
	float width;            // 
	string title;           // ^Cg
	public string Title {
		set{this.title = value;}
		get{return this.title;}
	}
	public float Width {
		set{this.width = value;}
		get{return this.width;}
	}
	public GraphItemStyle Style {
		set{this.style = value;}
		get{return this.style;}
	}
	public Color Color {
		set{this.color = value;}
		get{return this.color;}
	}
	public Point [] Points{
		set{this.points = value;}
		get{return this.points;}
	}
	public GraphItem(){
		style = GraphItemStyle.Lines;
		width = 1.0f;
	}
	
}

// Ot̕`X^C
// ̂܂܂̈Ӗ
public enum GraphItemStyle {Lines, Points, Box, LinesPoints};


// class GraphTest {
// 	public static void Main(){
// 		Form form = new Form();
// 		form.Size = new Size(800, 600);
// 		GraphPanel graph = new GraphPanel();
// 		graph.Dock = DockStyle.Fill;
// 		form.Controls.Add(graph);
// 		Application.Run(form);
// 	}
// }
